#ifndef IRCLEANER_H
#define IRCLEANER_H

#define ADJUST_THRESHOLD 0  /* raise auto thresholds to include more pixels */

#define NUM_GRAY 256		/**< Number of gray levels in an 8-bit gray-scale image */
#define MAX_GRAY 255		/**< Max. gray level in an 8-bit gray-scale image */

#define SWAP_INT( a, b ) { tmp_var = ( a ); ( a ) = ( b ); ( b ) = tmp_var; }

#define ERROR_RET( err_msg, err_no ) \
         do \
          { \
           fprintf ( stderr, "Error: %s\n", ( err_msg ) ); \
           fflush ( stderr ); \
           return ( err_no ) ; \
          } while ( 0 )

#define SAFE_LOG( x ) ( ( ( x ) > 0.0 ) ? log ( ( x ) ) : ( 0.0 ) )


typedef enum HistoType
{
 HISTO_INVALID = 0,	/**< invalid */
 HISTO_INT,		/**< unnormalized */
 HISTO_DBL		/**< normalized */
} HistoType;

typedef struct Histogram
{
 HistoType type;
 int num_bins;	      /**< Number of Bins */
 int num_pixels;      /**< Number of Pixels in the Source Image */
 union
 {
  int *int_data;	 /**< For HISTO_INT */
  double *double_data;	 /**< For HISTO_DBL */
 } data;
} Histogram;

typedef enum PixelType
{
 PIX_INVALID = 0,  /**< invalid */
 PIX_BIN,	   /**< binary */
 PIX_GRAY8,	   /**< gray-scale, 8 bits per sample */
 PIX_GRAY16,	   /**< gray-scale, 16 bits per sample */
 PIX_RGB8,	   /**< RGB color, 8 bits per sample */
 PIX_RGB16	   /**< RGB color, 16 bits per sample */
} PixelType;


typedef struct Image
{
 PixelType type;	/**< Pixel Type */
 int num_rows;		/**< Number of Rows */
 int num_cols;		/**< Number of Columns */
 union
  {			/**< For PIX_BIN, PIX_GRAY, PIX_RGB */
    uint8_t *sample8;
    uint16_t *sample16;
  } data;
} Image;


typedef struct Fill_Task
{
  unsigned int *idx_map, *dist_map;
  uint16_t **in_rgb;
  int num_rows, num_cols;
  int win_size;
  int dist_max;
  int failed;
  int planes_todo;
  int thread_no;
  int cores_avail;
} Fill_Task;



#endif
